VectorZ extensions for Nippy. They have been copy-pasted across many projects and have finally centralized here.
VectorZ extensions for Nippy. They have been copy-pasted across many projects and have finally centralized here.
(vectorz-get vector i)Get a single double value from AVector by index.
Get a single double value from AVector by index.
(vectorz? x)True if it is an instance of AVector.
True if it is an instance of AVector.
cljdoc builds & hosts documentation for Clojure/Script libraries
| Ctrl+k | Jump to recent docs |
| ← | Move to previous article |
| → | Move to next article |
| Ctrl+/ | Jump to the search field |